/*================================================================
  File:     03_Sorts.sql

  Summary:  A look at expensive sorts

  SQL Server Versions tested: SQL Server 2012, 2014
------------------------------------------------------------
  Written by Erin Stellato, SQLskills.com

  (c) 2014, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
=================================================================*/
USE [AdventureWorks2014];
GO

SET STATISTICS IO ON;
SET STATISTICS TIME ON;

/*
	SELECT query with no ORDER BY 
*/
SELECT [CustomerID], [SalesOrderID], [OrderDate], [SubTotal]
FROM [Sales].[SalesOrderHeaderEnlarged];


/*
	SELECT query with an ORDER BY 
*/
SELECT [CustomerID], [SalesOrderID], [OrderDate], [SubTotal]
FROM [Sales].[SalesOrderHeaderEnlarged] 
ORDER BY [CustomerID];


/*
	SELECT query with a TOP 
*/
SELECT TOP 10 [CustomerID], [SalesOrderID], [OrderDate], [SubTotal]
FROM [Sales].[SalesOrderHeaderEnlarged];

/*	
	SELECT query with a TOP, and an ORDER BY...
	because people usually want to see the data in some order, right?
*/
SELECT TOP 10 [CustomerID], [SalesOrderID], [OrderDate], [SubTotal]
FROM [Sales].[SalesOrderHeaderEnlarged]
ORDER BY [SalesOrderID] DESC;

/*
	SELECT query with a TOP 
*/
SELECT TOP 10 [CustomerID], [SalesOrderID], [OrderDate], [SubTotal]
FROM [Sales].[SalesOrderHeaderEnlarged]
ORDER BY [SubTotal] DESC;


/*
	Regular query with ORDER BY
*/
SELECT [CustomerID], [SalesOrderID], [OrderDate], [SubTotal]
FROM [Sales].[SalesOrderHeaderEnlarged] 
WHERE [CustomerID] < 12000 
ORDER BY [CustomerID];



/*
	What if I force the index?
*/
SELECT [CustomerID], [SalesOrderID], [OrderDate], [SubTotal]
FROM [Sales].[SalesOrderHeaderEnlarged] WITH (INDEX (IX_SalesOrderHeaderEnlarged_CustomerID))
WHERE [CustomerID] < 12000 
ORDER BY [CustomerID];


/*
	run together to look at statistics
*/
SET STATISTICS IO ON;

SELECT [CustomerID], [SalesOrderID], [OrderDate], [SubTotal]
FROM [Sales].[SalesOrderHeaderEnlarged] 
WHERE [CustomerID] < 12000 
ORDER BY [CustomerID];

SELECT [CustomerID], [SalesOrderID], [OrderDate], [SubTotal]
FROM [Sales].[SalesOrderHeaderEnlarged] WITH (INDEX (IX_SalesOrderHeaderEnlarged_CustomerID))
WHERE [CustomerID] < 12000 
ORDER BY [CustomerID];

SET STATISTICS IO OFF;


/*
	Add a covering index to support the query
*/
CREATE NONCLUSTERED INDEX [IX_SalesOrderHeaderEnlarged_CustomerID_OrderDate_SubTotal] ON [Sales].[SalesOrderHeaderEnlarged](
	[CustomerID] ASC
	) 
	INCLUDE (
	[OrderDate], [SubTotal]
	)
	WITH (
	PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, 
	ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON
	) ON [PRIMARY];
GO


/*
	SELECT query with an ORDER BY, now with an index to support
*/
SET STATISTICS IO ON
SELECT [CustomerID], [SalesOrderID], [OrderDate], [SubTotal]
FROM [Sales].[SalesOrderHeaderEnlarged]
WHERE [CustomerID] < 12000 
ORDER BY [CustomerID];
SET STATISTICS IO OFF

/*
	for tuning's sake, what's the IO improvement?
*/
SET STATISTICS IO ON;

SELECT [CustomerID], [SalesOrderID], [OrderDate], [SubTotal]
FROM [Sales].[SalesOrderHeaderEnlarged] WITH (INDEX (PK_SalesOrderHeaderEnlarged_SalesOrderID))
WHERE [CustomerID] < 12000 
ORDER BY [CustomerID];

SELECT [CustomerID], [SalesOrderID], [OrderDate], [SubTotal]
FROM [Sales].[SalesOrderHeaderEnlarged] WITH (INDEX (IX_SalesOrderHeaderEnlarged_CustomerID))
WHERE [CustomerID] < 12000 
ORDER BY [CustomerID];

SELECT [CustomerID], [SalesOrderID], [OrderDate], [SubTotal]
FROM [Sales].[SalesOrderHeaderEnlarged]
WHERE [CustomerID] < 12000 
ORDER BY [CustomerID];

SET STATISTICS IO OFF;


/*
	Remove our index
*/
DROP INDEX [IX_SalesOrderHeaderEnlarged_CustomerID_OrderDate_SubTotal] ON [Sales].[SalesOrderHeaderEnlarged]


/*
	A second query, looking at a lot more data
*/
SET STATISTICS IO ON;
SELECT [CustomerID], [SalesOrderID], [OrderDate], [SubTotal]
FROM [Sales].[SalesOrderHeaderEnlarged]
WHERE [OrderDate] BETWEEN '2012-01-01 00:00:00.000' AND '2013-12-31 23:59:59.997'
ORDER BY [OrderDate]
OPTION (RECOMPILE);


/*
	Check out row and page count
*/
SELECT 
	OBJECT_NAME([p].[object_id]) [TableName], 
	[si].[name] [IndexName], 
	[au].[type_desc] [Type], 
	[p].[rows] [RowCount], 
	[au].total_pages [PageCount]
FROM [sys].[partitions] [p]
JOIN [sys].[allocation_units] [au] ON [p].[partition_id] = [au].[container_id]
JOIN [sys].[indexes] [si] on [p].[object_id] = [si].object_id and [p].[index_id] = [si].[index_id]
WHERE [p].[object_id] = OBJECT_ID(N'Sales.SalesOrderHeaderEnlarged');


/*
	Trick the optimize a bit here and change row count
	**Don't try this at home!!**
*/
UPDATE STATISTICS  [Sales].[SalesOrderHeaderEnlarged]
	WITH ROWCOUNT = 100000, PAGECOUNT = 29883;
GO


/*
	re-run
*/
SELECT [CustomerID], [SalesOrderID], [OrderDate], [SubTotal]
FROM [Sales].[SalesOrderHeaderEnlarged]
WHERE [OrderDate] BETWEEN '2012-01-01 00:00:00.000' AND '2013-12-31 23:59:59.997'
ORDER BY [OrderDate]
OPTION (RECOMPILE);


/*
	clean up 
*/
UPDATE STATISTICS  [Sales].[SalesOrderHeaderEnlarged]
	WITH ROWCOUNT = 1290065, PAGECOUNT = 29883;
GO



